<?php
/**
* Used mainly in the chat logic to save messages in chat directories as files.
* It is also used to delete the same messages and folders.
*
* @filesource
* @package General_Classes
*/
/**
* Used mainly in the chat logic to save messages in chat directories as files.
* It is also used to delete the same messages and folders.
*
* @package General_Classes
*/
class FileDirExt{
	
	/**
	 * Returns an array with all filenames of the files in the directory
	 *
	 * @param dir &$d the directory to parse
	 * @return array the array with filenames
	 */
  	static function dirAsArr(&$d){
  	  	$arr = array();
		for($i = 0; false !== ($entry = $d->read()); $i++){
			if($i > 1) 	
				$arr[] = $entry;
		}

		return $arr;
	}
	
	/**
	 * Same functionality as dirAsArr() but accepts a string representing the
	 * directory name.
	 *
	 * @uses FileDirExt::dirAsArr()
	 * @param string $strDir the directory path
	 * @return array the array with filenames
	 */
	static function list_dir($strDir){
		$d = dir($strDir);
		return self::dirAsArr($d);
	}
	
	/**
	 * Lists all the files in a directory, ignores subdirectories.
	 * 
	 * @uses FileDirExt::dirAsArr() To get the filenames in an array
	 * @param string $strDir The directory path to look in, must have trailing /
	 */
	static function list_files_in_dir($strDir){
		$d = dir($strDir);
		$temp = self::dirAsArr($d);
		$rarr = array();
		foreach($temp as $item){
			if(is_file("$strDir/$item")){
				$rarr[] = $item;
			}
		}
		return $rarr;
	}
	
	/**
	 * Deletes everything within a directory through recursion.
	 *
	 * @uses FileDirExt::list_dir() to get all the contents of a directory
	 * @uses FileDirExt::make_dir_empty() uses itself to recursively delete subdirs
	 * @param string $strDir the directory path
	 */
	static function make_dir_empty($strDir){
		$arr = self::list_dir($strDir);
		foreach($arr as $item){
			$item = $strDir.$item;
			if(is_dir($item)){
				self::make_dir_empty($item."/");
				rmdir($item);
			}else if(is_file($item))
				unlink($item);
		}
	}
	
	/**
	 * Removes a directory completely.
	 *
	 * @uses FileDirExt::make_dir_empty() to delete everything inside the dir
	 * @param string $strDir the directory path
	 */
	static function remove_dir($strDir){
		if(is_dir($strDir)){
			self::make_dir_empty($strDir);
			rmdir($strDir);
		}
	}
	
	/**
	 * Returns a multi dimensional array containing all the lines in all files
	 * in a directory. Used during development to consolidate smarty config files.
	 *
	 * @uses FileDirExt::dirAsArr() to get all the files in the dir
	 * @param string $strDir the directory path
	 * @return array the resultant array
	 */
	static function dirAsLineArr($strDir){
		$d = dir($strDir);
  	  	$arr = self::dirAsArr($d);
		$rarr = array();
		foreach($arr as $file){
			$file_path = $strDir.$file;
			if(is_file($file_path)){
				$rarr[] = array("filename_path" => $file_path, "file_name" => $file, "content" => file($file_path));
			}
		}
		return $rarr;
	}
	
	/**
	 * Makes a string replacement in a file, it replaces all occurences of the
	 * needle with the replacement. 
	 *
	 * @param string $fileName The file name.
	 * @param string $needle The needle to look for.
	 * @param string $replacement The replacement string.
	 * @return array An array with each line as an element
	 */
	static function replaceInFile($fileName, $needle, $replacement){
	  	$lines = file($fileName);
	  	foreach($lines as &$line){
		    $line = str_replace($needle, $replacement, $line);
		}
		return $lines;
	}
	
	/**
	 * Erases a file and puts the contents of an array there.
	 *
	 * @param arr &$arr A reference to the array of lines (strings) to save to file.
	 * @param string $fileName The path to the file.
	 */
	static function emptyWrite1D(&$arr, $fileName){
	  	$handle = fopen($fileName, "w+");
	  	foreach($arr as &$line){
		    fwrite($handle, $line);
		}
		fclose($handle);
	}
	
	/**
	 * Replaces all occurances of a needle in all files in a dir with a replacement.
	 *
	 * @uses FileDirExt::dirAsArr() To fetch the whole dir into an array.
	 * @uses FileDirExt::emptyWrite1D() To save the files after changing them.
	 * @uses FileDirExt::replaceInFile() T handle the replacements in each file.
	 * @param dir &$dir The path to the dir to work on.
	 * @param string $needle The needle to look for.
	 * @param string $replacement The replacement.
	 */
	static function replaceInDir(&$dir, $needle, $replacement){
	  	$fileNames = self::dirAsArr($dir);
	  	foreach($fileNames as $fileName){
	  	  	$fileName = $dir->path."/".$fileName;
		    $fileAsArr = self::replaceInFile($fileName, $needle, $replacement);
		    self::emptyWrite1D($fileAsArr, $fileName);
		}
	}
}
?>