<?php
/**
* Used for common string operations.
* @filesource
* @package General_Classes
*/
/**
* Used for common string operations.
* @package General_Classes
*/
class StringManip{
  	/**
	* Converts a string to HTML.
	* @param string $str The string to convert.
	* @return string The converted string.
	*/
	static function toHtml($str){
		$str = stripslashes($str);
	  	$str = htmlentities($str, ENT_QUOTES, 'UTF-8');
	  	$str = nl2br($str);
		//$str = wordwrap($str,60,"<br>",1);
		//$str = self::whiteToHtml($str);
		//$str = nl2br($str);
		return $str;
	}
	
	/**
	* Converts a string to unformatted text.
	* @param string $str The string to convert.
	* @return string The converted string.
	* @uses StringManip::br_ForTextField_nl() to convert breaks that html_entity_decode() can't handle.
	*/
	static function toText($str){
	  	return self::br_ForTextField_nl(html_entity_decode(stripslashes($str), ENT_QUOTES, 'UTF-8'));
	}
	/**
	* To convert breaks that html_entity_decode() can't handle.
	* @param string $str The string to convert.
	* @return string The converted string.
	*/
	static function br_ForTextField_nl($str){
	  	$search[0] = '(<br />)';		$replace[0] = '';
		return preg_replace($search, $replace, $str);
	}
	/**
	* Validates a username that can't be empty or contain whitespaces.
	* @param string $username The string to check.
	* @return bool True if the string checks out ok, false otherwise.
	*/
	static function validateUserName($username){
	  	$arr = count_chars($username, 1);
	  	if(self::searchWhiteInByteCodeArr($arr))
	  		return false;
	  	
	  	return true; 	
	}
	
	/**
	* Validates a string by searching for whitespaces.
	* @param array &$arr The array with bytecodes.
	* @return bool True if the string doesn't check out (if none of the forbidden values between 0 and 33 are found), false otherwise.
	*/
	static function searchWhiteInByteCodeArr(&$arr){
	  	for($i = 0; $i < 33; $i++){
	  		if(isset($arr[$i]))
	  			return true;
		}
		return false;
	}
	
	/**
	 * Searches for a non white character in a byt code array.
	 *
	 * @param array $arr The array with byte codes.
	 * @return returns true if a non-white char was found, false otherwise
	 */
	static function searchNonWhiteInByteCodeArr(&$arr){
		foreach($arr as $key => $value){
			if($key > 32)
				return true;
		}
		return false;
	}

	/**
	 * Enter description here...
	 *
	 * @param unknown_type $str
	 * @return unknown
	 */
	static function checkForNonWhite($str){
		$arr = count_chars($str, 1);
		if(self::searchNonWhiteInByteCodeArr($arr))
			return 1;
		return 0;
	}
	
	/**
	 * Converts to html and strips unecessary white spaces and slashes, all quotes
	 * will be converted with the php htmlentities() function anyway.
	 * 
	 * @uses StringManip::whiteToHtml() To convert white spaces to html entities.
	 * @param string $str The string to work with.
	 * @return the finished string
	 */
	static function toHtmlWhite($str){
		$str = stripslashes($str);
	  	$str = htmlentities($str, ENT_QUOTES, 'UTF-8');
	  	$str = nl2br($str);
		$str = self::whiteToHtml($str);
		return $str;
	}

	/**
	 * Checks for special characters.
	 * 
	 * @param string $str The string to work with.
	 * @return false if an offending character was found, true otherwise
	 */
	static function validate_linux_sys($str){
		if($str == "")
			return 0;
		$str = strtolower($str);
		$allowed_str = "abcdefghijklmnopqrstuvwxyz0123456789_";
		for($i = 0; $i < strlen($str); $i++){
			if(strpos($allowed_str, $str{$i}) === false)
				return 0;
		}
		return 1;
	}
	
	/**
	 * Replaces all special characters with an underscore.
	 * 
	 * @param string $str The string to work with.
	 * @return the string with all special chars replaced with underscore
	 */
	static function to_linux_sys($str){
		if($str == "")
			return 0;
		$str = strtolower($str);
		$allowed_str = "abcdefghijklmnopqrstuvwxyz0123456789_";
		for($i = 0; $i < strlen($str); $i++){
			if(strpos($allowed_str, $str{$i}) === false)
				$str{$i} = '_';
		}
		return $str;
	}
	
	/**
	 * Fixes a whole array of strings by replacing special chars
	 * 
	 * @param array &$arr The array to work with.
	 * @uses StringManip::to_linux_sys() to make each and every replacement.
	 */
	static function arr_to_linux_sys(&$arr){
		foreach($arr as &$str)
			$str = self::to_linux_sys($str);
	}
	
	/**
	 * Removes all characters that are necessary for html injections. This function
	 * is used where the htmlentities function can not be used. (Pair name is one
	 * example)
	 * 
	 * @param string $str The string to work with.
	 */
	static function remove_html($str){
		$search[0] = '(<)';	$replace[0] = '';
		$search[1] = '(>)';	$replace[1] = '';
		$search[2] = '(/)';	$replace[2] = '';
		return preg_replace($search, $replace, $str);
	}
	
	/**
	 * Converts a field in a 2D array (table) to html.
	 *
	 * @param array &$arr The array to work with.
	 * @param string $key The key for the value to convert.
	 */
	static function to_html_arr_bykey(&$arr, $key){
		foreach($arr as &$sub){
			$sub[$key] = self::toHtml($sub[$key]);
		}
	}

	/**
	 * Converts white space combos to html equivalents.
	 * 
	 * @param string $str The string to work with.
	 */
	static function whiteToHtml($str){
		$search[0] = '( )';			$replace[0] = '&nbsp;';
		$search[1] = '(		)';		$replace[1] = '&nbsp;&nbsp;&nbsp;';
		return preg_replace($search, $replace, $str);
	}
	
	/**
	 * Returns an array with the positions of and the content inside
	 * tags of one kind inside a string.
	 * 
	 * @param string $s_tag The start tag.
	 * @param string $e_tag The end tag.
	 * @param string &$str The string to work with.
	 * @return array The array with the results. 
	 */
	static function get_tag_content($s_tag, $e_tag, &$str){
		$rarr = array();
		$str_count = substr_count($str,$s_tag);
		if($str_count == 0)
			return $rarr;
		else{
			$cur_pos = 0;
			for($i = 0; $i < $str_count; $i++){
				$s_pos = strpos($str,$s_tag,$cur_pos) + strlen($s_tag);
				$e_pos = strpos($str,$e_tag,$cur_pos);
				if($s_pos === false || $e_pos === false)
					break;
				$rarr[] = array(
					"content" 	=> substr($str, $s_pos, $e_pos - $s_pos),
					"start" 	=> $s_pos,
					"end" 		=> $e_pos
				);
				$cur_pos = $e_pos + strlen($e_pos);
			}
		}
		return $rarr;
	}
	
	static function extract($s_pattern, $e_pattern, &$str){
		$str_count = substr_count($str, $s_pattern);
		if($str_count == 0)
			return "";
		$start = strpos($str, $s_pattern) + strlen($s_pattern);
		$end = strpos($str, $e_pattern, $start);
		if($start === false || $end === false || $start < 0 || $end < 0 || $start == $end)
			return "";
		
		//echo $start."<br>";
		//echo $end."<br>";
		
		return substr($str,$start,$end-$start);
	}

	static function anyCharsInStrQ($haystack, $chars){
		for($i = 0; $i < strlen($chars); $i++){
			$char = $chars{$i};
			if(strpos($haystack,$char) !== false)
				return true;
		}
		return false;
	}
}
?>