<?php
include('FileDirExt.php');
include('StringManip.php');

/**
 * In and out folders, we do not overwrite the original code
 * only copy. First we have the folder to read from and the
 * respective folder to write to in position 2
 */
$folders = array(
	array("in" => "myproject/", "out" => "out/"),
	array("in" => "myproject/classes/", "out" => "out/classes/")
);

/**
 * The array that will contain non static relations. Essentially
 * a mapping of classes that call functions like this: $obj->function
 */
$relations = array();

/**
 * First we map all non static relations
 */
getRelations($folders, $relations);

/**
 * We loop through all the folders and pass all files in them
 * to mainLoop()
 */
foreach($folders as $sub){
	$file_arr = FileDirExt::dirAsLineArr($sub['in']);
	mainLoop($file_arr, $sub['out'], $relations);
}

/**
 * The main function of this little application.
 */
function mainLoop(&$file_arr, $out_folder, &$relations){
	foreach($file_arr as &$file){
		$file_cont =& $file['content'];

		$cur_dblock_intervals = getDocBlockIntervals($file_cont);
		if(count($cur_dblock_intervals) == 1){
			appendUsageTags(current($cur_dblock_intervals), count($file_cont) - 1, $file_cont, $relations);
		}else if(count($cur_dblock_intervals) > 1){
			for($i = 0; $i < count($cur_dblock_intervals); $i++){
				if($i < count($cur_dblock_intervals) - 1)
					appendUsageTags($cur_dblock_intervals[$i], $cur_dblock_intervals[$i+1]['start'], $file_cont, $relations);
				else
					appendUsageTags($cur_dblock_intervals[$i], count($file_cont) - 1, $file_cont, $relations);
			}
		}
		$file_str = implode($file_cont,"");

		file_put_contents($out_folder.$file['file_name'], $file_str);
	}
}

/**
 * Gets all the classes declared in all folders passed to this function.
 */
function getClassNames($folders){
	$class_names = array();
	foreach($folders as $sub){
		$file_arr = FileDirExt::dirAsLineArr($sub['in']);
		foreach($file_arr as $sub){
			$cur_classname = getClassName($sub['content']);
			if($cur_classname != false)
				$class_names[] = $cur_classname;
		}
	}
	return $class_names;
}

/**
 * Calls getNonStaticRelations with the help of an array that contains all the files
 * in each folder.
 */
function getRelations($folders, &$relations){
	foreach($folders as $sub){
		$file_arr = FileDirExt::dirAsLineArr($sub['in']);
		$relations = array_merge($relations, getNonStaticRelations($file_arr));
	}
	
}

/**
 * We map instances of objects by checking for the new word, we return an array
 * with all class names used to make instances.
 */
function getNonStaticRelations(&$file_arr){
	$relations = array();
	foreach($file_arr as &$file){
		$content =& $file['content'];
		foreach($content as $line){
			if(strpos($line," new") !== false && preg_match('/[\*\/]/',$line) == 0){
				$words = preg_split('/[$&=> ();\t]/', $line, -1, PREG_SPLIT_NO_EMPTY);
				$new_pos = array_search("new", $words);
				$var_name = $words[$new_pos - 1];
				$class_name = $words[$new_pos + 1 ];
				$relations[$var_name] = $class_name;
			}
		}
	}
	return $relations;
}

/**
 * Performs the actual writing of the new @uses blocks.
 */
function appendUsageTags($cur_dblock, $end, &$file_cont, &$relations){
	$usage = array();
	$start = $cur_dblock['end'];
	getStaticUsage($file_cont, $start, $end, $usage);
	getDynamicUsage($file_cont, $start, $end, $relations, $usage);
	$append_line =& $file_cont[$cur_dblock['end'] - 1];
	$padding = substr($append_line, 0, strpos($append_line, "*"));
	$usage_tags = assembleUsageTags($usage, $padding);
	$append_line .= $usage_tags;
}

/**
 * Puts together all the classes and respective functions or variables.
 */
function assembleUsageTags(&$usage, $padding){
	$usage_tags = "";
	foreach($usage as $class => $functions){
		$class = trim($class);
		foreach($functions as $function => $value){
			$function = trim($function);
			if($class != ""){
				if($value == "function")
					$usage_tags .= $padding."* @uses $class::$function()\n";
				else if($value == "variable"){
					if(strpos($function,"$") !== false)
						$usage_tags .= $padding."* @uses $class::$function\n";
					else
						$usage_tags .= $padding."* @uses $class::$$function\n";
				}
					
			}
		}
	}
	$usage_tags .= $padding."*\n";
	return $usage_tags;
}

/**
 * Returns the classname in a file, note that only one name per file can
 * be handled.
 */
function getClassName(&$file){
	if(determineFileType($file) == "class"){
		foreach($file as $line){
			if(strpos($line,"class ") !== false && preg_match('/[\*\/]/',$line) == 0){
				$words = preg_split('/[\t \/!{]/', $line, -1, PREG_SPLIT_NO_EMPTY);
				return $words[1];
			}
		}
	}
	
	return false;
}

/**
 * Checks if the file contains a class declaration or not.
 */
function determineFileType(&$file){
	foreach($file as $line){
		if(strpos($line,"class ") !== false && preg_match('/[\*\/]/',$line) == 0){
			return "class";
		}
	}
	return "proc";
}

/**
 * Calculates which line numbers contain doc blocks.
 */
function getDocBlockIntervals(&$file){
	$intervals = array();
	$i = 0;
	$inside_docblock = false;
	$temp = array();
	foreach($file as $line){
		if(strpos($line,"/**") !== false){
			$inside_docblock = true;
			$temp['start'] = $i;
		}
		if(strpos($line,"*/") !== false && $inside_docblock == true){
			$inside_docblock = false;
			$temp['end'] = $i;
			$intervals[] = $temp;
			$temp = array();
		}
		$i++;
	}
	return $intervals;
}

/**
 * Maps usage of static functions, returns an array with the class names used and the type
 * used, function or variable.
 */
function getStaticUsage(&$file, $start, $end, &$usage){
	for($i = $start; $i < $end; $i++){
		$kotayem_pos = strpos($file[$i],"::");
		$doubleslash_pos = strpos($file[$i],"//") === false ? 1000000 : strpos($file[$i],"//");
		if($kotayem_pos !== false && $doubleslash_pos > $kotayem_pos){
			$pieces = preg_split('/[\[\].=>();,\/\'!]/', $file[$i], -1, PREG_SPLIT_NO_EMPTY);
			foreach($pieces as $piece){
				$sub_pieces = explode("::",$piece);
				if(count($sub_pieces) == 2){
					$class_name = trim($sub_pieces[0]);
					if($class_name == "self"){
						$class_name = getClassName($file);
					}
					$function = trim($sub_pieces[1]);
					if(strpos($sub_pieces[1],'$') !== false)
						$usage[$class_name][$function] = "variable";
					else
						$usage[$class_name][$function] = "function";
				}
			}
		}
	}
}

/**
 * Maps usage of dynamic variables and functions, returns an array with the class names
 * used.
 */
function getDynamicUsage(&$file, $start, $end, &$relations, &$usage){
	for($i = $start; $i < $end; $i++){
		$arrow_pos = strpos($file[$i],"->");
		$doubleslash_pos = strpos($file[$i],"//") === false ? 1000000 : strpos($file[$i],"//");
		if($arrow_pos !== false && $doubleslash_pos > $arrow_pos){
			$pieces = preg_split('/[$.=;,\/\'"!]/', $file[$i], -1, PREG_SPLIT_NO_EMPTY);
			foreach($pieces as $piece){
				if(strpos($piece,"->") !== false){
					if(strpos($piece,"(") !== false)
						$type = "function";
					else
						$type = "variable";
					$piece = preg_replace('/[\[\]\(\){ ]/',"",$piece);
					$sub_pieces = explode("->",$piece);
					if(count($sub_pieces) == 2){
						$function = trim($sub_pieces[1]);
						$var_name = trim($sub_pieces[0]);
					}else if(count($sub_pieces) > 2){
						$func_pos = count($sub_pieces) - 1;
						$var_pos = $func_pos - 1;
						$function = trim($sub_pieces[$func_pos]);
						$var_name = trim($sub_pieces[$var_pos]);
					}
					if($var_name == "this"){
						$class_name = getClassName($file);
					}else{
						$class_name = $relations[$var_name];
					}
					$usage[$class_name][$function] = $type;
				}
			}
		}
	}
}

?>